@@ -14,7 +14,7 @@ except ImportError:
 import __builtin__
 
 
-__version__ = '1.2.4'
+__version__ = '1.2.5'
 __author__ = 'Yoshida Tetsuya'
 __license__ = 'MIT License'
 
@@ -49,12 +49,15 @@ class PyXdebug(object):
         self.call_func_name = getattr(func, '__name__', None)
         return self._run(func, *args, **kwds)
 
-    def run_statement(self, statement):
+    def run_statement(self, statement, globals_=None, locals_=None):
         self.initialize()
-        import __main__
-        dict = __main__.__dict__
+        if globals_ is None:
+            import __main__
+            globals_ = __main__.__dict__
+        if locals_ is None:
+            locals_ = inspect.currentframe().f_back.f_locals
         def exec_statement():
-            exec statement in dict, dict
+            exec statement in globals_, locals_
         return self._run(exec_statement)
 
     def run_file(self, script_path, globals_=None):
@@ -74,12 +77,12 @@ class PyXdebug(object):
             original_import = __builtin__.__import__
             original_reload = __builtin__.reload
 
-            def __pyxdebug_import_hook(name, globals=None, locals=None, fromlist=None):
+            def __pyxdebug_import_hook(name, globals=None, locals=None, fromlist=None, **kwds):
                 frame = inspect.currentframe()
                 self.trace_import(frame, (name, fromlist))
                 result = None
                 try:
-                    result = original_import(name, globals, locals, fromlist)
+                    result = original_import(name, globals, locals, fromlist, **kwds)
                     return result
                 finally:
                     self.trace_return(frame, result)
@@ -141,7 +144,13 @@ class PyXdebug(object):
 
         # wrap frame
         frame = FrameWrap(frame)
-        frame.f_back = FrameWrap(frame.f_back)
+        f_back = frame.f_back
+        while f_back:
+            if os.path.splitext(os.path.abspath(f_back.f_code.co_filename))[0]==this_path:
+                f_back = f_back.f_back
+            else:
+                break
+        frame.f_back = FrameWrap(f_back)
 
         # dispatch call
         if event=='call':
@@ -181,15 +190,15 @@ class PyXdebug(object):
             self.result.append(trace)
 
     def trace_line(self, frame, arg):
-        back_frame = self.late_dispatch[self.call_depth-1]
+        pre_frame = self.late_dispatch[self.call_depth-1]
         self.late_dispatch[self.call_depth-1] = frame
 
-        if back_frame:
+        if pre_frame:
             frame = FrameWrap(frame)
-            frame.set_position(back_frame)
-            self._trace_line(frame, None)
+            frame.set_position(pre_frame)
+            self._trace_line(frame)
 
-    def _trace_line(self, frame, arg):
+    def _trace_line(self, frame):
         line = frame.get_line().strip()
         match = re.compile(r"^([^\+\-\*/=]+)([\+\-\*/]?=[^=])(.+)$").match(line)
         if match:
@@ -205,7 +214,7 @@ class PyXdebug(object):
             if varnames:
                 for varname in varnames:
                     value = get_frame_var(frame, varname)
-                    trace = SubstituteTrace(frame, self.call_depth)
+                    trace = AssignmentTrace(frame, self.call_depth)
                     trace.setvalue(varname, value)
                     self.result.append(trace)
 
@@ -230,7 +239,7 @@ class PyXdebug(object):
         return result
 
 
-class AbstractTrace(object):
+class BaseTrace(object):
     def __init__(self, callee, call_depth):
         if callee:
             self.callee = callee
@@ -241,7 +250,7 @@ class AbstractTrace(object):
         self.call_depth = call_depth
 
 
-class CallTrace(AbstractTrace):
+class CallTrace(BaseTrace):
     def __init__(self, callee, call_depth):
         super(CallTrace, self).__init__(callee, call_depth)
         self.time = None
@@ -303,18 +312,18 @@ class CallTrace(AbstractTrace):
         return u'%10.4f %10d   %s-> %s(%s) %s:%d' % (self.time or 0.0, self.memory or 0, sp, self.callee_name(), params, self.caller_filename(), self.caller_lineno())
 
 
-class ReturnTrace(AbstractTrace):
-    def setvalue(self, ret_value):
-        self.ret_value = ret_value
+class ReturnTrace(BaseTrace):
+    def setvalue(self, value):
+        self.value = value
 
     def get_result(self):
         sp = u' '*24 + u'  '*self.call_depth
-        return u'%s>=> %s' % (sp, pformat(self.ret_value))
+        return u'%s>=> %s' % (sp, pformat(self.value))
 
 
-class SubstituteTrace(AbstractTrace):
+class AssignmentTrace(BaseTrace):
     def __init__(self, callee, call_depth):
-        super(SubstituteTrace, self).__init__(callee, call_depth)
+        super(AssignmentTrace, self).__init__(callee, call_depth)
         self.varname = None
         self.value = None
 
@@ -374,7 +383,7 @@ class FinishTrace(CallTrace):
         return u'%10.4f %10d' % (self.time or 0.0, self.memory or 0)
 
 
-class LogTrace(AbstractTrace):
+class LogTrace(BaseTrace):
     def __init__(self, callee, call_depth):
         super(LogTrace, self).__init__(callee, call_depth)
         self.message = None
